# -*- coding: utf-8 -*-

import re
import pickle
import csv
import os
import json

#            0          1               2                  3                          4                   5                       6                       7                 8                          9                  10                     11               12                     13            14                     15                      16                   17                     18                      19                       20                21                           22                23
websites = ['buy.net', 'cammarkt.com', 'www.alibaba.com', 'www.buzzillions.com', 'www.cambuy.com.au', 'www.camerafarm.com.au', 'www.canon-europe.com', 'www.ebay.com', 'www.eglobalcentral.co.uk', 'www.flipkart.com', 'www.garricks.com.au', 'www.gosale.com', 'www.henrys.com', 'www.ilgs.net', 'www.mypriceindia.com', 'www.pcconnection.com', 'www.price-hunt.com', 'www.pricedekho.com', 'www.priceme.co.nz', 'www.shopbot.com.au', 'www.shopmania.in', 'www.ukdigitalcameras.co.uk', 'www.walmart.com', 'www.wexphotographic.com']

alias = [('vivitar', 'sakar'),
         ('cannon', 'canon'),
         ('cyber-shot', 'cybershot'),
         ('cyber shot', 'cybershot'),
         ('fujifilm ', 'fuji '),
         ('digital ixus', 'ixus'),
         (' mk ii', ' mark ii'),
         ('canon a', 'canon powershot a'),
         (' is', ' '),
         (' dmc-', ' '),
         (' dmc', ' '),
         (' dsc-', ' '),
         (' dsc', ' '),
         (' dslr-', ' '),
         (' dslr', ' '),
         ('(', ''),
         (')', ''),
         (',', ' , '),
         ('v³', 'vpow3'),
         ('pro 1', 'pro1'),
         ('nex ', 'nex'),
         ('-', ''),
         ('&', '')]

re_alias = [(r'(\d)\s?ihs', r'\1'),
            (r'(\d\d)\s?is', r'\1'),
            (r'(\d\d)\s?hsr', r'\1'),
            (r'(\d\d)\s?hs', r'\1'),
            (r'(\d\d)\s?exr', r'\1'),
            (r'(\d)\s?gps', r'\1'),
            (r'(\d\d)\s?uz', r'\1'),
            (r'(\d\d)\s?fd', r'\1'),
            (r'(\d\d)\s?hd', r'\1'),
            (r'(\d\d)\s?v', r'\1'),
            (r'(\d)\s?q', r'\1'),
            (r'(\d)\s?lens style', r'\1'),
            (r'stylus 1$', r'sty1xyz'),
            (r'general electric ', r'ge ')]

can_remove_list = ['cybershot',
                   'powershot',
                   'finepix',
                   'lumix',
                   'stylus',
                   'alpha',
                   'multiview',
                   'galaxy',
                   'ilce',
                   'type',
                   'typ',
                   'easyshare',
                   'optio',
                   'tough',
                   'slt',
                   'instax',
                   'coolpix',
                   'exilim',
                   'pen',
                   '1',
                   'eos',
                   'mju']

def create_dataframe(dataset_path):
    print('>>> Creating dataframe...\n')
    df = []
    i = 0
    for source in os.listdir(dataset_path):
        for specification in os.listdir(os.path.join(dataset_path, source)):
            specification_number = specification.replace('.json', '')
            with open(os.path.join(dataset_path, source, specification)) as specification_file:
                specification_data = json.load(specification_file)
                specification_data['source'] = source
                specification_data['specification_number'] = specification_number
                specification_data['ID'] = i
                df.append(specification_data)
                i += 1
    print('>>> Dataframe created successfully!\n')
    return df

def replace_alias(brand):
    for s1, s2 in alias:
        brand = brand.replace(s1, s2)
    for s1, s2 in re_alias:
        brand = re.sub(s1, s2, brand)
    return brand

def is_equal2(m1, m2):
    s1 = set(m1[0].split())
    s2 = set(m2[0].split())
    s3 = s1.symmetric_difference(s2)

    if len(s3) <= 0:
        return True
    return False

def add_to_known(brands, brand, ID):
    brand = replace_alias(brand)
    brand = ' '.join([i.strip() for i in brand.split()])
    max_group = -1
    to_add = [brand, ID]
    for m in brands:
        if is_equal2(m, to_add):
            max_group = m[2]-1
            break
        else:
            max_group = max(max_group, m[2])

    max_group += 1
    to_add = [brand, ID, max_group]
    brands.append(to_add)
    return brands



def add_model(brands, brand, ID):

    brands = add_to_known(brands, brand, ID)
    return brands


def grep_title(data, source, tags, brands=[]):
    if type(tags) != list:
        tags = [tags]
    t = '<page title>'
    for i, item in enumerate(data):
        if t in item:
            if item['source'] == source:
                brand = item[t].lower()
                for tag in tags:
                    i = brand.find(tag)
                    if i >= 0:
                        brand = brand[:i-1]

                brand = brand.strip()
                ID = item['ID']
                brands = add_model(brands, brand, ID)
    return brands


def print_brands(brands_list, file_name):
    with open(file_name, 'w') as fid:
        for b in brands_list:
            if type(b) == list:
                fid.write(', '.join(b) + '\n')
            else:
                fid.write(' '.join(b.split()) + '\n')

def read_equal_list():
    equal_list = []
    file_name = r'equal.txt'
    with open(file_name, 'r', encoding='utf-8') as fid:
        reader = csv.reader(fid, delimiter=',')
        for row in reader:
            stripped = [i.strip() for i in row]
            equal_list.append(stripped)
    return equal_list


def combine_equal(brands):
    brands.sort(key=lambda tup: tup[-1], reverse=False)

    brands_set = set()
    key = -1
    for b in brands:
        if key != b[2]:
            brands_set.add(b[0])
            key = b[2]

    equal_list = read_equal_list()
    for equal in equal_list:
        for item in equal:
            if not item in brands_set:
                brands_set.add(item)

    brands_list = list(brands_set)
    brands_list.sort()
    combined_list = []
    for brand in brands_list:
        if brand in brands_set:
            new_camera_list = [brand]
            brands_set.remove(brand)

            for cl in equal_list:
                if brand in cl:
                    for c in cl:
                        if c in brands_set:
                            new_camera_list.append(c)
                            brands_set.remove(c)

            combined_list.append(new_camera_list)
    return combined_list


def all_in(a, b):
    title_s = set(b.split())
    if a <= title_s:
        return True
    return False

def in_brands(item, brands_set, p=False):
    title = item['<page title>'].lower()
    title = replace_alias(title)
    found = False

    for b in brands_set:
        if all_in(b, title):
            found = True
            break
    if found:
#        print('.', end='')
        return True
    else:
#        print('-', end='')
        return False


def collect_ebay(data, brands):
    brands_set = set(sum(brands, []))
    can_remove = set(can_remove_list)
    brands_set_set = []
    for b in brands_set:
        brands_set_set.append(set(b.split()).difference(can_remove))

    s = {}
    total = 0
    has = 0
    for item in data:
        num = item['ID']
        if item['source'] == 'www.ebay.com':
            if in_brands(item, brands_set_set):
                continue
            total += 1
            if 'brand' in item and 'model' in item:
                has += 1
                brand = item['brand']
                if type(brand) == list:
                    brand = brand[0]
                mdl = item['model']
                if type(mdl) == list:
                    mdl = mdl[0]
                brand = brand.lower()
                mdl = mdl.lower()
                model = brand + ' ' + mdl
                model = replace_alias(model)
                if model in s:
                    s[model].append(num)
                else:
                    s[model] = [num]
    ebay = []
    for i, e in s.items():
        if len(e) > 18:
            ebay.append(i)
    ebay.sort()
    return ebay


if __name__ == '__main__':
    dataset_path = '../predict/2013_camera_specs'

    data = create_dataframe(dataset_path)
    # for faster reading
#    pickle.dump(data, open('specs.pkl', 'wb'))
#    data = pickle.load(open('specs.pkl', 'rb'))


    print('creating labels_a ...', end='')
    brands = []
    # shopmania
    brands = grep_title(data, websites[20], ['digital camera', 'kit', 'body', '+'])
    # canon
    brands = grep_title(data, websites[6], '-', brands)
    # wex
    brands = grep_title(data, websites[23], ['digital', 'instant', 'white', 'black', 'pink'], brands)
    # uk
    brands = grep_title(data, websites[21], ['digital', 'slr', 'camera', '(', 'black', '+'], brands)
    # india
    brands = grep_title(data, websites[14], ['price in', '(', 'inter', 'cam', 'live'], brands)
    # priceme
    brands = grep_title(data, websites[18], ['camera', 'new zealand', '+', 'digital'], brands)
    # shopbot
    brands = grep_title(data, websites[19], ['- price', '/', 'with'], brands)

    brands.sort(key=lambda tup: tup[0])
    brands_full = brands.copy()

    equal_list = read_equal_list()
    brands = combine_equal(brands)
    print(' done')

    print('creating labels_b ...', end='')
    ebay = collect_ebay(data, brands)
    print(' done')

    print_brands(brands, 'labels_a.txt')
    print_brands(ebay, 'labels_b.txt')
