#include "cleaner.h"
#include <algorithm>
#include <cstring>
#include <stdexcept>

inline void replace_space_out(std::string &s, const std::string &search_for,
                              const std::string &replace_with) {
  if (replace_with.size() > search_for.size()) {
    throw std::runtime_error("size of <replace_with> must be <= size of <search_for>");
  }

  char *pch = const_cast<char *>(strstr(s.data(), search_for.data()));

  while (pch != nullptr) {
    strncpy(pch, replace_with.data(), replace_with.size());
    for (size_t i = 0; i < search_for.size() - replace_with.size(); ++i) {
      pch[i + replace_with.size()] = ' ';
    }
    pch = const_cast<char *>(strstr(pch + replace_with.size(), search_for.data()));
  }
}

inline void pad_tolower_alphanum(std::string &line) {
  // to lowercase
  std::transform(line.begin(), line.end(), line.begin(),
                 [](unsigned char c) { return std::tolower(c); });
  // at line[0] and line[line.size() - 1] are quotes, make them spaces
  line[0] = ' ';
  line[line.size() - 1] = ' ';
  // keep only lowercase alpha numeric and -
  for (size_t j = 1; j < line.size() - 1; ++j) {
    if ((line[j] >= 'a' && line[j] <= 'z') ||
        (line[j] >= '0' && line[j] <= '9')) {
    } else if (line[j] == '-') {
      if (line[j - 1] == ' ' || line[j + 1] == ' ') {
        line[j] = ' ';
      }
    } else {
      line[j] = ' ';
    }
  }
}

inline void clean_uni_code(std::string &line) {
  replace_space_out(line, "\\u03b1", "alpha");
  replace_space_out(line, "\\u00b3", "pow3");
}

inline void remove_hyphen(std::string &line) {
  size_t j = 0;
  for (size_t i = 0; i < line.size(); ++i) {
    while (line[i] == '-') {
      ++i;
    }
    if (line[i] != '\0') {
      line[j++] = line[i];
    }
  }
  for (; j < line.size(); ++j) {
    line[j] = ' ';
  }
}

inline void clean_title(std::string &line) {
  replace_space_out(line, "vivitar", "sakar");
  replace_space_out(line, "cannon", "canon");
  remove_hyphen(line);
  replace_space_out(line, "cyber shot", "cybershot");
  replace_space_out(line, "fujifilm ", "fuji ");
  replace_space_out(line, "digital ixus", "ixus");
  replace_space_out(line, " is", " ");
  replace_space_out(line, " dmc", " ");
  replace_space_out(line, " dsc", " ");
  replace_space_out(line, " dslr", " ");

  // extra
  replace_space_out(line, " d lux ", " dlux ");
  replace_space_out(line, " t 100 ", " t100 ");
  replace_space_out(line, " mx 1 ", " mx1 ");
  replace_space_out(line, " v lux ", " vlux ");
  replace_space_out(line, " om d ", " omd ");
  replace_space_out(line, " x t1 ", " xt1 ");
  replace_space_out(line, " x t1 ", " xt1 ");
  replace_space_out(line, " x 42 ", " x42 ");
  replace_space_out(line, " x a1 ", " xa1 ");
  replace_space_out(line, " x m1 ", " xm1 ");
  replace_space_out(line, " x s1 ", " xs1 ");
  replace_space_out(line, " x e1 ", " xe1 ");
  replace_space_out(line, " x 5 ", " x5 ");
  replace_space_out(line, " ex z", "  exz");
  replace_space_out(line, " ex t", "  ext");
  replace_space_out(line, " ex s", "  exs");

  replace_space_out(line, " vg ", "  vg");
  replace_space_out(line, " fe ", "  fe");
  replace_space_out(line, " k ", "  k");
  replace_space_out(line, " vh ", "  vh");
  replace_space_out(line, " sz ", "  sz");
  replace_space_out(line, " vr ", "  vr");
  replace_space_out(line, " xz ", "  xz");
  replace_space_out(line, " sh ", "  sh");

  // aggressive
  replace_space_out(line, " tg ", "  tg");
  replace_space_out(line, " e ", "  e");
  replace_space_out(line, " wg ", "  wg");
  replace_space_out(line, " sp ", "  sp");
  replace_space_out(line, " nex ", "  nex");

  // typos and rubbish
  replace_space_out(line, " xblack ", " x ");
  replace_space_out(line, " g1 x ", " g1x ");
  replace_space_out(line, "hd ", " ");
  replace_space_out(line, " 2 7 ", " ");
  replace_space_out(line, " canonpowershot ", " canon ");
  replace_space_out(line, " condtion n ", " ");
  replace_space_out(line, " aw 120 ", " aw120 ");
  replace_space_out(line, " repairsony ", " sony ");
  replace_space_out(line, " 100    12 ", " ");
  replace_space_out(line, " sd 1000 ", " sd1000 ");
  replace_space_out(line, " alpha 7 r ", " alpha 7r ");
  replace_space_out(line, " f3 55 6 ii ", " ");
  replace_space_out(line, " sd 900 ", " sd900 ");
  replace_space_out(line, " sd 790 ", " sd790 ");
  replace_space_out(line, " fd 87 ", " fd87 ");
  replace_space_out(line, " d 53 ", " d53 ");
  replace_space_out(line, " nicon ", " nikon ");
  replace_space_out(line, " power shot ", " powershot ");
  replace_space_out(line, " fugifilm ", " fuji ");
  replace_space_out(line, " fugi ", " fuji ");
  replace_space_out(line, " fine pix ", " finepix ");
  replace_space_out(line, " go pro ", " gopro ");
  replace_space_out(line, " 10 1 mp  3 ", " ");
  replace_space_out(line, " leica lens ", " ");
  replace_space_out(line, " 6 3 mp ", " ");
  replace_space_out(line, " 6 5 mp ", " ");
  replace_space_out(line, " with 3 0 inch ", " ");
  replace_space_out(line, " slr 6 3 ", " ");
  replace_space_out(line, " leica x 1 ", " leica x1 ");
  replace_space_out(line, " evf 3 1 of 2 ", " ");
  replace_space_out(line, " 1 mp ", " ");
  replace_space_out(line, " only 3500 shots ", " ");
  replace_space_out(line, " f3 5 5 6 ", " ");
  replace_space_out(line, " 2 mp ", " ");
  replace_space_out(line, " using leica m lens ", " ");
  replace_space_out(line, " w150 v1 ", " w150 ");
  replace_space_out(line, " kit w v1 10 ", " ");
  replace_space_out(line, " black 3 lithium ion ", " ");
  replace_space_out(line, " w650 v1 ", " w650 ");
  replace_space_out(line, " w100 v1 ", " w100 ");
  replace_space_out(line, " w170 v1 ", " w170 ");
  replace_space_out(line, " rebel t 3 ", " rebel t3 ");
  replace_space_out(line, " camera 580ex ii ", " ");
  replace_space_out(line, " 775 v1 ", " 775 ");
  replace_space_out(line, " w5 v1 ", " w5 ");
  replace_space_out(line, " 42mm f3 5 4 6 ", " ");
  replace_space_out(line, " f 1 4 ", " ");
  replace_space_out(line, " nixon ", " nikon ");
  replace_space_out(line, " u0091nex 5n ", " nex5n ");
  replace_space_out(line, " canon ds 6041 slr rebel ", " canon eos ds6041 ");
  replace_space_out(line, " canon rebel 6 millions pixels ", " canon eos ds6041 ");
  replace_space_out(line, " ek gc110 ", " ekgc110 ");
  replace_space_out(line, " ekgc200z", " ekgc200 ");
  replace_space_out(line, " ek gc200 ", " ekgc200 ");
  replace_space_out(line, " samsung galaxy ekgc110 ", " samsunggalaxyekgc110 ");
  replace_space_out(line, " ek gc120 ", " ekgc120 ");
  replace_space_out(line, " battery grip 64gb 75 300mm t5i t4i t3i t2i ", " ");
  replace_space_out(line, " canon 60d digital slr 013803129052 ", " canon eos 60da ");
  replace_space_out(line, " 5 7 17 1mm 1 2 8 3 6 ", " ");
  replace_space_out(line, " fuijifilm ", " fuji ");
  replace_space_out(line, " fujifilem ", " fuji ");
  replace_space_out(line, " fuji film ", " fuji ");
  replace_space_out(line, " fuji x pro 1 ", " fuji xpro1 ");
  replace_space_out(line, " kinon ", " nikon ");

  // full text
  replace_space_out(line, " coolpix a ", " coolpixa ");
  replace_space_out(line, " stylus 1 ", " sty1xyz ");
  replace_space_out(line, " olympus style 1 ", " sty1xyz ");
  replace_space_out(line, " olympus stylus 120megapixel ", " sty1xyz ");
  replace_space_out(line, " olympus stylus 1a ", " sty1xyz ");
}

// for the labels file
void preprocess_label(std::string &line) {
  replace_space_out(line, "vivitar", "sakar");
  replace_space_out(line, "fujifilm", "fuji");
  remove_hyphen(line);
  replace_space_out(line, " nex ", "  nex");
}

inline void replace_string(std::string &result, const std::string &original, const std::string &newstr){
  auto pos = result.find(original);
  if(pos != std::string::npos){
    result.replace(std::begin(result) + pos, std::begin(result) + pos + original.size(), newstr);
  }
}

inline void clear_out(std::string &result, const std::string &with){
  auto posw = result.find(with);
  if(posw != std::string::npos){
    for (size_t i = posw; i < result.size(); ++i) {
      result[i] = ' ';
    }
  }
}

inline bool is_num(unsigned char c){
  return c < 58 && c > 47;
}

// hardcoded regex
inline int space_out_count(const char* s){
  //  ihs|is|hsr|hs|exr|gps|uz|fd
  int c = 0;
  if(s[0] == 'i'){
    if(s[1] == 's'){
      c = 2;
    } else if(s[1] == 'h' && s[2] == 's'){
      c = 3;
    }
  } else if(s[0] == 'h'){
    if(s[1] == 's'){
      c = 2;
      if(s[2] == 'r'){
        c++;
      }
    }
  } else if(s[0] == 'e' && s[1] == 'x' && s[2] == 'r'){
    c = 3;
  } else if(s[0] == 'g' && s[1] == 'p' && s[2] == 's'){
    c = 3;
  } else if(s[0] == 'u' && s[1] == 'z') {
    c = 2;
  } else if(s[0] == 'f' && s[1] == 'd') {
    c = 2;
  }
  return c;
}

inline void mock_regex(std::string &line){
  for(size_t i = 0; i < line.size() - 1; ++i){
    int count = 0;
    while(is_num(line[i])){
      i++;
      count++;
    }
    if(count > 1 && i < line.size() - 3){
      int c = space_out_count(line.data() + i);
      while(c--){
        line[i++] = ' ';
      }
    }
  }
}

inline std::string regex_clean(std::string &line) {
  mock_regex(line);

  static const std::string eos1d = " eos1d ";
  static const std::string eos1dn = " eos 1d ";
  replace_string(line, eos1d, eos1dn);

  static const std::string eos7d = " eos7d ";
  static const std::string eos7dn = " eos 7d ";
  replace_string(line, eos7d, eos7dn);

  static const std::string mkii = " mkii ";
  static const std::string mkiin = " mark ii ";
  replace_string(line, mkii, mkiin);

  static const std::string mk = " mk ";
  static const std::string mkn = " mark ";
  replace_string(line, mk, mkn);

  static const std::string eos1dx = " xmark ";
  static const std::string eos1dxn = " x mark ";
  replace_string(line, eos1dx, eos1dxn);

  static const std::string with = "shopping comparison";
  clear_out(line, with);

  static const std::string price = "price in india";
  clear_out(line, price);

  static const std::string digital = "digital   point";
  clear_out(line, digital);

  static const std::string sale = "on sale for";
  clear_out(line, sale);

  static const std::string lowepro = "lowepro";
  clear_out(line, lowepro);

  return line;
}

inline bool starts_with(const std::string &line, const std::string &start) {
  size_t i = 0;
  for (; i < line.size() && line[i] == ' '; ++i);
  for (size_t j = 0; j < start.size() && i < line.size(); ++j, ++i) {
    if (start[j] != line[i]) {
      return false;
    }
  }
  return i != line.size() || i != line.size() + 1;
}

// test if label is a kit
bool kit(const std::string &line){
  if(starts_with(line, "tripod kit"))
    return true;
  if(starts_with(line, "accessory kit"))
    return true;
  if(starts_with(line, "battery kit"))
    return true;
  if(starts_with(line, "gripster kit"))
    return true;
  if(starts_with(line, "deluxe accessory kit"))
    return true;
  if(starts_with(line, "advanced accessory kit"))
    return true;
  return false;
}

void preprocess_title(std::string &line){
  clean_uni_code(line);
  pad_tolower_alphanum(line);
  clean_title(line);
  line = regex_clean(line);
}
