#pragma once

#include "camera.h"
#include <algorithm>
#include <fstream>
#include <stdexcept>
#include <string>
#include <sys/types.h> // required for stat.h
#include <sys/stat.h>
#include <unordered_map>
#include <utility>

struct Label_Debbuger {
  std::string buffer_cameras_found;
  std::string buffer_cameras_ambiguous;
  std::string buffer_cameras_not_found;
  const std::string dir_debug;
  std::unordered_map<int, std::unordered_map<std::string, int>> labels{};

  static void makedir(const std::string& path_to_dir) {
    int nError = 0;
#if defined(_WIN32)
    nError = mkdir(path_to_dir.c_str()); // Windows MinGW (on MSVC-Compiler use _mkdir)
#else
    mode_t nMode = 0733; // UNIX style permissions
    nError = mkdir(path_to_dir.c_str(), nMode); // non-Windows
#endif
    if (nError != 0) {
      throw std::runtime_error("cannot create " + path_to_dir);
    }
  }

  static bool dir_exist(const std::string &path_to_dir){
    struct stat info{};
    return stat(path_to_dir.c_str(), &info) == 0;
  }

  explicit Label_Debbuger(std::string _dir_debug, std::vector<Result_Pair>* vec_debug = nullptr) : dir_debug(std::move(_dir_debug)) {
    if(!dir_exist(dir_debug)){
      makedir(dir_debug);
    }
    buffer_cameras_found.reserve(100000);
    buffer_cameras_ambiguous.reserve(100000);
    buffer_cameras_not_found.reserve(100000);
    if(vec_debug != nullptr && !(*vec_debug).empty()){
      std::sort(std::begin(*vec_debug), std::end(*vec_debug), [](const Result_Pair& a, const Result_Pair& b){
        return a.record.id < b.record.id;
      });
      for(const auto& debug_pair : *vec_debug){
        update(debug_pair.record, debug_pair.cameras);
      }
    }
  }

  void update(const Record& r, const std::vector<Camera>& cameras){
    if(cameras.size() == 1){
      buffer_cameras_found += r.id + "\n";
      buffer_cameras_found += "page title: " + r.title + "\n";
      for (const auto& cam : cameras) {
        buffer_cameras_found += "(ID " + std::to_string(cam.model_id) + "): " + cam.internal_label + "\n";
      }
      buffer_cameras_found += "\n";

      for(const auto& cam : cameras){
        labels[cam.model_id][cam.internal_label]++;
      }
    } else if (cameras.size() > 1){
      buffer_cameras_ambiguous += r.id + "\n";
      buffer_cameras_ambiguous += "page title: " + r.title + "\n";
      for (const auto& cam : cameras) {
        buffer_cameras_ambiguous += "(ID " + std::to_string(cam.model_id) + "): " + cam.internal_label + "\n";
      }
      buffer_cameras_ambiguous += "\n";
    }else{
      buffer_cameras_not_found += r.id + "\n";
      buffer_cameras_not_found += "page title: " + r.title + "\n";
      buffer_cameras_not_found +=  "\n";
    }
  }

  void write_buffers_to_disk(){
    // save cameras found
    std::ofstream ofs_found("./" + dir_debug + "/_cameras_found.txt");
    ofs_found << buffer_cameras_found << std::endl;

    // save ambiguous
    std::ofstream ofs_ambiguous("./" + dir_debug +  "/_cameras_ambiguous.txt");
    ofs_ambiguous << buffer_cameras_ambiguous << std::endl;

    // save not found
    std::ofstream ofs_not_found("./" + dir_debug +  "/_cameras_not_found.txt");
    ofs_not_found << buffer_cameras_not_found << std::endl;

    // save labels found
    std::ofstream ofs_labels("./" + dir_debug +  "/_labels_found_sorted.txt");
    std::vector<std::string> str_labels;
    for (auto const& x : labels){
      std::string line;
      std::string cleaned_line;
      std::string tmp_line;
      int items = 0;
      for (const auto& s: x.second) {
        tmp_line += s.first + ", ";
        items += s.second;
      }
      if (tmp_line.size() > 1) {
        line = std::to_string(100000 + items).substr(2) + " (ID " + std::to_string(x.first) + "): " +
               tmp_line.substr(0, tmp_line.size() - 2);
      }
      size_t i = 0;
      while(i < line.size()){
        while(line[i] == ' ' && i < line.size()){
          ++i;
        }
        while(line[i] != ' ' && i < line.size()){
          cleaned_line += line[i];
          ++i;
        }
        if(i < line.size()){
          cleaned_line += ' ';
        }
      }
      str_labels.push_back(cleaned_line);
    }
    std::sort(begin(str_labels), end(str_labels), std::greater<std::string>());
    for(const auto& s: str_labels){
      ofs_labels << s << "\n";
    }
  }
};
