#include "io.h"
#include "cleaner.h"
#include <cstring>
#include <dirent.h>
#include <fstream>
#include <iostream>
#include <stdexcept>
#include <sstream>

std::vector<std::string> ls(const std::string &path) {
  std::vector<std::string> entries;
  entries.reserve(1000);
  auto *dp = opendir(path.data());
  if (dp == nullptr) {
    throw std::runtime_error("path \"" + path + "\" does not exist");
  }
  dirent *entry;
  while ((entry = readdir(dp))) {
    if (!strcmp(entry->d_name, ".") || !strcmp(entry->d_name, "..")) {
    } else {
      entries.emplace_back(entry->d_name);
    }
  }
  closedir(dp);
  return entries;
}

std::vector<Record> get_records(const std::string &path_to_camera_directory) {
  auto entries = ls(path_to_camera_directory);

  const std::string title = "<page title>\": ";
  std::vector<Record> records;
  records.reserve(30000);

  for (const auto &entry : entries) {
    // alibaba contains only trash
    if(entry.find("alibaba.com") != std::string::npos){
      continue;
    }
    auto cameras = ls(path_to_camera_directory + entry);
    std::string tmp = path_to_camera_directory + entry + "/";
#pragma omp parallel num_threads(16)
    {
      std::vector<Record> local_records;
#pragma omp for schedule(dynamic)
      for (size_t i = 0; i < cameras.size(); ++i) {
        std::ifstream fin(tmp + cameras[i]);
        std::string line;
        bool flag = true;
        while (flag && getline(fin, line)) {
          auto position = line.find(title);
          if (position != std::string::npos) {
            position = position + title.size();
            // extract string between ': "' and '",' quotes are included
            line = line.substr(position, line.size() - position - 1);
            // clean title
            preprocess_title(line);
            // not a kit
            if(!kit(line)){
              std::string id = entry + "//" + cameras[i].substr(0, cameras[i].size() - 5);
              local_records.push_back({id, line});
            }
            flag = false;
          }
        }
        fin.close();
      }
#pragma omp critical
      records.insert(records.end(), local_records.begin(), local_records.end());
    }
  }
  return records;
}

size_t to_csv(const std::vector<std::vector<std::string>> &buckets) {
  std::ofstream csv("submission.csv");
  csv << "left_spec_id,right_spec_id\n";
#pragma omp parallel
  {
    std::string matching_pairs_tmp;
    matching_pairs_tmp.reserve(10000);
#pragma omp for schedule(dynamic)
    for (size_t b = 0; b < buckets.size(); ++b) {
      matching_pairs_tmp.clear();
      for (int64_t i = 0; i < int64_t(buckets[b].size()) - 1; ++i) {
        for (int64_t j = i + 1; j < int64_t(buckets[b].size()); ++j) {
          matching_pairs_tmp += buckets[b][i] + ',' + buckets[b][j] + '\n';
        }
      }
#pragma omp critical
      if (!matching_pairs_tmp.empty()) {
        csv << matching_pairs_tmp;
      }
    }
  }
  return csv.tellp();
}

std::vector<Camera> read_labels(const std::string &path_to_labels, int id) {
  std::ifstream fin(path_to_labels);
  if(fin.fail()){
    throw std::runtime_error("labels file missing \"" + path_to_labels + "\"");
  }
  std::vector<Camera> cameras;
  std::string line;
  std::string model;
  std::string word;
  line.reserve(1000);
  model.reserve(200);
  word.reserve(100);
  while (getline(fin, line)) {
    if (!line.empty()) {
      std::stringstream ss_line(line);
      while (ss_line.good()) {
        getline(ss_line, model, ',');
        if(!model.empty()){
          preprocess_label(model);
          std::stringstream ss_model(model);
          Camera cam{};
          while (ss_model >> word) {
            cam.toke_set.insert(word);
          }
          cam.model_id = id;
          cam.internal_label = model;
          cameras.emplace_back(cam);
        }
      }
      id++;
    }
  }
  return cameras;
}