#include "io.h"
#include "debugger.h"
#include "Matcher.h"
#include <chrono>
#include <iostream>
#include <iomanip>

#define DEBUG false

// helper macros for timing
#define TIMERSTART(label)                                                      \
  std::chrono::time_point<std::chrono::system_clock> a##label, b##label;       \
  a##label = std::chrono::system_clock::now();

#define TIMERSTOP(label)                                                       \
  b##label = std::chrono::system_clock::now();                                 \
  std::chrono::duration<double> delta##label = b##label - a##label;            \
  std::cout << #label << ": " << std::setprecision(4) << delta##label.count()  \
            << " seconds" << std::endl;

int main() {
  std::string path_camera_specs = "./2013_camera_specs/";
  std::string path_labels = "./labels/";

  TIMERSTART(total);
  TIMERSTART(init_labels);
  Matcher::init(path_labels);
  TIMERSTOP(init_labels);

  TIMERSTART(parse_and_clean_json_cameras);
  auto records = get_records(path_camera_specs);
  TIMERSTOP(parse_and_clean_json_cameras);

  TIMERSTART(match);
  std::vector<Result_Pair> matched_cameras(records.size());
#pragma omp parallel for
  for (size_t i = 0; i < records.size(); ++i) {
    auto &r = records[i];
    std::vector<Camera> cameras = Matcher::match(r);
    matched_cameras[i] = {std::move(cameras), r};
  }

  std::vector<std::vector<std::string>> buckets(Matcher::size());
  int found = 0;
  // based on model ids of cameras throw them into buckets
  // same camera model is put into same bucket
  for(const auto& result_pair : matched_cameras){
    // unambiguous result (only one camera found)
    if(result_pair.cameras.size() == 1){
      found++;
      buckets[result_pair.cameras[0].model_id].push_back(result_pair.record.id);
    }
  }
  TIMERSTOP(match);

  TIMERSTART(write_csv);
  auto csv_size = to_csv(buckets);
  TIMERSTOP(write_csv);

  if(DEBUG){
    TIMERSTART(produce_debug_info);
    Label_Debbuger label_debugger{"_debug_info", &matched_cameras};
    label_debugger.write_buffers_to_disk();
    TIMERSTOP(produce_debug_info);
  }

  std::cout << std::string(40, '-') << std::endl;
  TIMERSTOP(total);
  std::cout << "amount of cameras matched successfully: " << found << std::endl;
  std::cout << "submission.csv generated (size: " <<  csv_size << " bytes)" << std::endl;
}
